﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Web;
using System.Data;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Text.RegularExpressions;
using System.Collections;

using CMS.GlobalHelper;
using CMS.FormControls;
using CMS.FormEngine;
using CMS.IDataConnectionLibrary;

public partial class CMSFormControls_Custom_RelatedDropDown : FormEngineUserControl
{
    #region "Private variables"

    private static Regex mRegRelated = null;
    private static Regex mRegSQL = null;
    private ArrayList mRelatedControlNames = null;
    private object mFormControlParameter = null;
    private string mSelectedValue = "";

    #endregion


    #region "Private properties"


    /// <summary>
    /// Regular expression for parsing field settings to get related dropdown controls.
    /// </summary>
    private static Regex RegRelated
    {
        get
        {
            if (mRegRelated == null)
            {
                mRegRelated = RegexHelper.GetRegex("<related>([^<]*)</related>");
            }
            return mRegRelated;
        }
    }


    /// <summary>
    /// Regular expression for parsing field settings to get SQL query.
    /// </summary>
    private static Regex RegSQL
    {
        get
        {
            if (mRegSQL == null)
            {
                mRegSQL = RegexHelper.GetRegex("<sql>([^<]*)</sql>");
            }
            return mRegSQL;
        }
    }

    #endregion


    #region "Public porperties"

    /// <summary>
    /// Gets or sets the enabled state of the control
    /// </summary>
    public override bool Enabled
    {
        get
        {
            return base.Enabled;
        }
        set
        {
            base.Enabled = value;
            this.drpRelated.Enabled = value;
        }
    }


    /// <summary>
    /// Gets or sets related control names.
    /// </summary>
    public ArrayList RelatedControlNames
    {
        get
        {
            if (mRelatedControlNames == null)
            {
                mRelatedControlNames = new ArrayList();
            }
            return mRelatedControlNames;
        }
        set
        {
            mRelatedControlNames = value;
        }
    }


    /// <summary>
    /// Gets or sets field value.
    /// </summary>
    public override object Value
    {
        get
        {
            return drpRelated.SelectedValue;
        }
        set
        {
            EnsureItems();
            drpRelated.SelectedValue = ValidationHelper.GetString(value, "");
            mSelectedValue = ValidationHelper.GetString(value, "");
        }
    }


    /// <summary>
    /// Helper parameter - ID of selected item of parent control 
    /// </summary>
    public override object FormControlParameter
    {
        get
        {
            return mFormControlParameter;
        }
        set
        {
            mFormControlParameter = value;

            if (mFormControlParameter != null)
            {
                Match matchRelatedSQL = RegSQL.Match(this.FieldInfo.Description.ToLower());
                // Get SQL query text
                string sqlText = matchRelatedSQL.Groups[0].Value.ToLower().Replace("<sql>", "").Replace("</sql>", "");

                object[,] parameter = new object[1, 3];

                parameter[0, 0] = "@ID";
                parameter[0, 1] = ValidationHelper.GetString(mFormControlParameter, "");

                DataSet ds = null;

                try
                {
                    ds = this.Form.Connection.ExecuteQuery(sqlText, parameter, QueryTypeEnum.SQLQuery, false);
                }
                catch { }

                if (!DataHelper.DataSourceIsEmpty(ds))
                {
                    // Show current form control
                    this.Visible = true;
                    drpRelated.Enabled = true;
                    Label lblField = (Label)this.Form.FieldLabels[this.FieldInfo.Name];
                    lblField.Visible = true;

                    drpRelated.Items.Clear();
                    drpRelated.DataSource = ds;
                    drpRelated.DataValueField = ds.Tables[0].Columns[0].ColumnName;
                    drpRelated.DataTextField = ds.Tables[0].Columns[1].ColumnName;
                    drpRelated.SelectedValue = ds.Tables[0].Rows[0][0].ToString();

                    // Set selected value if possible
                    if (!string.IsNullOrEmpty(mSelectedValue))
                    {
                        bool match = false;

                        foreach (DataRow item in ds.Tables[0].Rows)
                        {
                            if (mSelectedValue.Equals(ValidationHelper.GetString(item[0], "")))
                            {
                                match = true;
                            }
                        }

                        if (match)
                        {
                            drpRelated.SelectedValue = mSelectedValue;
                        }
                    }

                    drpRelated.DataBind();
                }
                else
                {
                    // Hide current form control
                    this.Visible = false;
                    Label lblField = (Label)this.Form.FieldLabels[this.FieldInfo.Name];
                    lblField.Visible = false;
                    this.drpRelated.Items.Clear();
                }
            }
        }
    }


    /// <summary>
    /// Gets ClientID of the DropDownList
    /// </summary>
    public override string ValueElementID
    {
        get
        {
            return drpRelated.ClientID;
        }
    }

    #endregion


    #region "Protected methods"

    /// <summary>
    /// OnAfterDataLoad BasicForm handler
    /// </summary>
    protected void Form_OnAfterDataLoad()
    {
        // Check if related control is specified and exists
        MatchCollection matchRelated = RegRelated.Matches(this.FieldInfo.Description);
        bool match = false;

        foreach (Match item in matchRelated)
        {
            if (item.Success)
            {
                string controlFieldName = item.Groups[1].Value;
                FormEngineUserControl drpMatch = (FormEngineUserControl)this.Form.FieldControls[controlFieldName];

                if (drpMatch != null)
                {
                    match = true;
                    drpMatch.FormControlParameter = drpRelated.SelectedValue; // SiteID
                    // Add matched field name to arraylist
                    RelatedControlNames.Add(controlFieldName);
                }
            }
        }

        if (match)
        {
            drpRelated.AutoPostBack = true;
            drpRelated.SelectedIndexChanged += new EventHandler(drpRelated_SelectedIndexChanged);
        }

        else
        {
            drpRelated.AutoPostBack = false;
        }
    }
    

    /// <summary>
    /// SelectedIndexChanged DropDownList event
    /// </summary>
    protected void drpRelated_SelectedIndexChanged(object sender, EventArgs e)
    {
        if (RelatedControlNames.Count > 0)
        {
            foreach (string controlName in RelatedControlNames)
            {
                FormEngineUserControl drpMatch = (FormEngineUserControl)this.Form.FieldControls[controlName];
                drpMatch.FormControlParameter = drpRelated.SelectedValue; // SiteID
            }
        }
    }


    /// <summary>
    /// Loads DropDownList values
    /// </summary>
    protected void EnsureItems()
    {
        if (drpRelated.Items.Count == 0)
        {
            // Check if SQL query is specified and execute it only in case of main DropDownList - not related
            Match matchSQL = RegSQL.Match(this.FieldInfo.Description);

            if (matchSQL.Success)
            {
                // Get SQL query text
                string sqlText = matchSQL.Groups[1].Value;

                if (!sqlText.Contains("@"))
                {
                    DataSet ds = null;

                    try
                    {
                        ds = this.Form.Connection.ExecuteQuery(sqlText, null, QueryTypeEnum.SQLQuery, false);
                    }
                    catch { }

                    if (!DataHelper.DataSourceIsEmpty(ds))
                    {
                        // Initialize control
                        drpRelated.DataSource = ds;
                        drpRelated.DataValueField = ds.Tables[0].Columns[0].ColumnName;
                        drpRelated.DataTextField = ds.Tables[0].Columns[1].ColumnName;
                        drpRelated.DataBind();

                        // Register OnAfterDataLoad handler
                        this.Form.OnAfterDataLoad += new BasicForm.OnAfterDataLoadEventHandler(Form_OnAfterDataLoad);
                    }
                }
            }
        }
    }


    /// <summary>
    /// OnPreRender override
    /// </summary>
    protected override void OnPreRender(EventArgs e)
    {
        base.OnPreRender(e);

        // Remove field Description - contains SQL query text
        Label lblField = (Label)this.Form.FieldLabels[this.FieldInfo.Name];
        if (lblField != null)
        {
            string toolTip = lblField.Text;

            // Remove colon if used
            if (this.Form.UseColonBehindLabel)
            {
                toolTip = toolTip.Remove(toolTip.Length - 1);
            }

            lblField.ToolTip = toolTip;
        }
    }

    #endregion
}